@@ -12,6 +12,7 @@ import os
 
 from tqdm import tqdm
 from concurrent.futures import ThreadPoolExecutor
+from bamt.utils import GraphUtils as gru
 from pyvis.network import Network
 
 from typing import Dict, Tuple, List, Callable, Optional, Type, Union, Any, Sequence
@@ -128,7 +129,7 @@ class BaseNetwork(object):
         nodes must be a subclass of BaseNode.
         :param nodes dict with name and node (if a lot of nodes should be added)
         """
-        if not info and not self.descriptor["types"]:
+        if not info and not (self.descriptor["types"] and self.descriptor["signs"]):
             logger_network.error("In case of manual setting nodes user should set map for them as well.")
             return
         self.nodes = []
@@ -190,7 +191,7 @@ class BaseNetwork(object):
                 builder.get_family()
                 if self.edges:
                     builder.overwrite_vertex(has_logit=self.has_logit, use_mixture=self.use_mixture)
-                    self.set_nodes(nodes=builder.skeleton['V'])
+                    self.set_nodes(builder.vertices)
                 else:
                     logger_network.error("Empty set of edges")
 
@@ -237,34 +238,6 @@ class BaseNetwork(object):
             json.dump(self.edges, out)
         return True
 
-    def save(self, outdir: str):
-        """
-        Function to save the whole BN to json file
-        :param outdir: output directory
-        """
-        if not outdir.endswith('.json'):
-            return None
-        outdict = {
-            'info': self.descriptor,
-            'edges': self.edges,
-            'parameters': self.distributions
-        }
-        with open(outdir, 'w+') as out:
-            json.dump(outdict, out)
-        return True
-
-    def load(self, input_dir: str):
-        """
-        Function to load the whole BN from json file
-        :param input_dir: input directory
-        """
-        with open(input_dir) as f:
-            input_dict = json.load(f)
-
-        self.add_nodes(input_dict['info'])
-        self.set_structure(edges=input_dict['edges'])
-        self.set_parameters(parameters=input_dict['parameters'])
-
     def fit_parameters(self, data: pd.DataFrame, dropna: bool = True):
         """
         Base function for parameters learning
@@ -288,6 +261,12 @@ class BaseNetwork(object):
             columns_names = [name for name, t in self.descriptor['types'].items() if t in ['disc_num']]
             data[columns_names] = data.loc[:, columns_names].astype('str')
 
+        # Topology sorting
+        ordered = gru.toporder(self.nodes, self.edges)
+        notOrdered = [node.name for node in self.nodes]
+        mask = [notOrdered.index(name) for name in ordered]
+        self.nodes = [self.nodes[i] for i in mask]
+
         def worker(node):
             return node.fit_parameters(data)
 
@@ -318,21 +297,15 @@ class BaseNetwork(object):
                 print(
                     f"{n.name: <20} | {n.type: <50} | {self.descriptor['types'][n.name]: <10} | {str([self.descriptor['types'][name] for name in n.cont_parents + n.disc_parents]): <50} | {str([name for name in n.cont_parents + n.disc_parents])}")
 
-    def sample(self,
-               n: int,
-               evidence: Optional[Dict[str, Union[str, int, float]]] = None,
-               as_df: bool = True,
-               predict: bool = False,
-               parall_count: int = 1) -> \
-            Union[None, pd.DataFrame, List[Dict[str, Union[str, int, float]]]]:
+    def sample(self, n: int, evidence: Optional[Dict[str, Union[str, int, float]]] = None,
+               as_df: bool = True, predict: bool = False) -> Union[
+        None, pd.DataFrame, List[Dict[str, Union[str, int, float]]]]:
         """
         Sampling from Bayesian Network
         n: int number of samples
         evidence: values for nodes from user
-        parall_count: number of threads. Defaults to 1.
         """
-        from joblib import Parallel, delayed
-
+        seq = []
         random.seed()
         if not self.distributions.items():
             logger_network.error("Parameter learning wasn't done. Call fit_parameters method")
@@ -343,51 +316,40 @@ class BaseNetwork(object):
                     if not (isinstance(evidence[node.name], str)):
                         evidence[node.name] = str(int(evidence[node.name]))
 
-        def wrapper(bn, evdnce):
+        for n in range(n):
             output = {}
-            for node in bn.nodes:
+            for node in self.nodes:
                 parents = node.cont_parents + node.disc_parents
-                if evdnce:
-                    if node.name in evdnce.keys():
-                        output[node.name] = evdnce[node.name]
+                if evidence:
+                    if node.name in evidence.keys():
+                        output[node.name] = evidence[node.name]
                     else:
                         if not parents:
                             pvals = None
                         else:
-                            if bn.type == 'Discrete':
+                            if self.type == 'Discrete':
                                 pvals = [str(output[t]) for t in parents]
                             else:
                                 pvals = [output[t] for t in parents]
                         if predict:
-                            output[node.name] = \
-                                node.predict(
-                                    bn.distributions[node.name], pvals=pvals)
+                            output[node.name] = node.predict(self.distributions[node.name], pvals=pvals)
                         else:
-                            output[node.name] = \
-                                node.choose(
-                                    bn.distributions[node.name], pvals=pvals)
+                            output[node.name] = node.choose(self.distributions[node.name], pvals=pvals)
 
                 else:
                     if not parents:
                         pvals = None
                     else:
-                        if bn.type == 'Discrete':
+                        if self.type == 'Discrete':
                             pvals = [str(output[t]) for t in parents]
                         else:
                             pvals = [output[t] for t in parents]
                     if predict:
-                        output[node.name] = \
-                            node.predict(
-                                bn.distributions[node.name], pvals=pvals)
+                        output[node.name] = node.predict(self.distributions[node.name], pvals=pvals)
                     else:
-                        output[node.name] = \
-                            node.choose(
-                                bn.distributions[node.name], pvals=pvals)
-            return output
+                        output[node.name] = node.choose(self.distributions[node.name], pvals=pvals)
 
-        seq = Parallel(n_jobs=parall_count)(
-            delayed(wrapper)(self, evidence)
-            for i in tqdm(range(n), position=0, leave=True))
+            seq.append(output)
 
         if as_df:
             return pd.DataFrame.from_dict(seq, orient='columns')
@@ -407,10 +369,6 @@ class BaseNetwork(object):
         Returns:
             predicted data (dict): dict with column as key and predicted data as value
         """
-        if test.isnull().any().any():
-            logger_network.error("Test data contains NaN values.")
-            return {}
-
         from joblib import Parallel, delayed
 
         def wrapper(bn: HybridBN, test: pd.DataFrame, columns: List[str]):
