@@ -27,7 +27,7 @@ This module provides tools for creating and using factorized representations of
 
 '''
 
-from external.libpgm.tablecpdfactor import TableCPDFactor
+from bamt.external.libpgm.tablecpdfactor import TableCPDFactor
 
 import random
 import copy
