@@ -7,7 +7,7 @@ References
 network structure learning algorithm."
 
 """
-from external.pyBN.learning.structure.hybrid.mmpc import mmpc
+from bamt.external.pyBN.learning.structure.hybrid.mmpc import mmpc
 
 def mmhc(data, alpha=0.05, metric='AIC', max_iter=100, method='hc'):
 	"""
